##
## Delete a single doc that has an invalid version
## Notice the routing= param as that is required when the version is invalid (i think)
##
delete_single_doc() {
    curl -s -XDELETE --header "Content-Type:application/json" -u "${ES_USER}:${ES_PASSWORD}" "https://${ES_HOST}:443/asset/_doc/${DOC_ID}?routing=${ORG_ID}" | jq
}


do_cleanup() {
    # 1. create file of org-ids on server
    if [[ -f "/tmp/${ENV}-deregistered-org-ids" ]]; then
        echo "Cleaning up data on ENV=${ENV}"
        echo "Using deregistered Org IDs file at /tmp/${ENV}-deregistered-org-ids with $(wc -l /tmp/${ENV}-deregistered-org-ids) entries"
    else 
        echo "Deregistered Org IDs file not found at: /tmp/${ENV}-deregistered-org-ids"
    fi

    # 2. get count of devicestats entries
    while read -r line; do count_devicestats_for_org_id $line; done < /tmp/${ENV}-deregistered-org-ids

    # 3. sort
    sort -n -k 2 /tmp/${ENV}-deregistered-org-ids-counts > /tmp/${ENV}-deregistered-org-ids-sorted
    grep -v ' 0' /tmp/${ENV}-deregistered-org-ids-sorted > /tmp/${ENV}-deregistered-org-ids-filtered
    echo "Found $(wc -l  /tmp/${ENV}-deregistered-org-ids-filtered) orgs with devicestats data"

    # 4. batch process
    echo "Starting cleanup"
    cleanup_by_query
}

count_entries_for_org() {
    local orgid=$1
    local index=${2:"a_devicestats"}

    curl -s --header "Content-Type:application/json" --url "$(cbd es)/$index/_count?q=organizationId:$orgid" | jq .count
}

##
## Deletes entries from the index for the given orgID within a time range
##
delete_in_range() {
    local orgid=$1
    local start=$2
    local end=$3
    local INDEX=a_devicestats

    curl -s -XPOST --header "Content-Type:application/json" --url $(cbd es)/$INDEX/_delete_by_query -d "
        {
        \"query\": { 
            \"bool\": { 
            \"must\": [
                { \"term\":  { \"organizationId\": $orgid }},
                {
                    \"range\" : {
                        \"createTime\" : {
                            \"gte\" : $start,
                            \"lte\": $end
                        }
                    }
                }
            ]
            }
        }
        }
        " | jq
}

##
## Deletes entries from the index for about a given number of months
## Starting from the earliest createTime
##
delete_by_months() {
    local orgid=$1
    local INDEX=a_devicestats

    # ~1 month
    local months=$(( 1 * 2678400000 ))
    # ~7 days
    # local months=$(( 1 * 624960000 ))

    local latest=$(curl -s -XPOST --header "Content-Type:application/json" --url "$(cbd es)/$INDEX/_search?size=1&q=organizationId:$orgid&sort=createTime:desc" | jq -r .hits.hits[]._source.createTime)
    local earliest=$(curl -s -XPOST --header "Content-Type:application/json" --url "$(cbd es)/$INDEX/_search?size=1&q=organizationId:$orgid&sort=createTime:asc" | jq -r .hits.hits[]._source.createTime)

    local start=$(( earliest ))
    local end=$(( start + months ))

    delete_in_range $orgid $start $end
}
